(** HEADER quickdraw_pascal.h
**)

const
   QD__MAX_STRING_SIZE = 128;

type
   Point = record
      x, y : integer;
   end;

   Rect = record
      top : integer;
      left : integer;
      bottom : integer;
      right : integer;
   end;

   Polygon = record
      num_vertices : integer;
      vertices : ^Point;
   end;

Patterns = (
   white,
   ltGray,
   gray,
   dkGray,
   black);

Colors = (
   whiteColor,
   blackColor,
   redColor,
   greenColor,
   blueColor,
   magentaColor,
   cyanColor,
   yellowColor);

PenModes = (
   patCopy,
   patXor);

Fonts = (
   FONT__SMALL,
   FONT__MEDIUM,
   FONT__LARGE,
   FONT__LARGEST);

LineStyles = (

   LINE_STYLE__SOLID,
   LINE_STYLE__ON_OFF_DASH,
   LINE_STYLE__DOUBLE_DASH);


#ifndef STRING_HEADER
#include "/pro/cs011/include/mstrings.h"
#endif


procedure InitDraw(width, height : integer); external c;
procedure ClearAll; external c;
procedure DrawGrid(spacing : integer) ; external c;
procedure GetDrawingRect(var r : Rect); external c;
procedure QuitDraw; external c;

procedure SetPt(var pt : Point; x, y : integer); external c;
procedure SetRect(var rect : Rect; left, top, right, bottom : integer); external c;
procedure OffsetRect(var rect : Rect; dh, dv : integer);external c;
procedure InsetRect(var rect : Rect; dh, dv : integer);external c;
function  PtInRect(point : Point; rect : Rect): boolean;external c;

procedure SetColor(color : Colors) ;external c;
procedure PenPat(pat : Patterns) ;external c;

function  Random:integer;external c;
procedure Randomize(seed : integer); external c;
procedure WaitDraw(t:integer);external c;

function Button: Boolean;external c;
procedure GetMouse(var x, y : integer); external c;
procedure GetMousePt(var pt : Point); external c;

procedure SetLineWidth(lineWidth : integer);external c;
procedure GetLineWidth(var lineWidth : integer);external c;
procedure SetLineStyle(style : LineStyles);external c;
procedure GetLineStyle(var style: LineStyles);external c;
procedure PenMode(mode : PenModes);external c;
procedure GetPenMode(var mode : PenModes);external c;
procedure SetFont(font : Fonts);external c;
procedure GetFont(var font : Fonts);external c;

procedure DrawLine(p1, p2 : Point);external c;
procedure EraseLine(p1, p2 : Point);external c;
procedure FrameRect(rect : Rect);external c;
procedure PaintRect(rect : Rect);external c;
procedure InvertRect(rect : Rect);external c;
procedure EraseRect(rect : Rect);external c;
procedure FrameOval(rect : Rect);external c;
procedure PaintOval(rect : Rect);external c;
procedure InvertOval(rect : Rect);external c;
procedure EraseOval(rect : Rect);external c;
procedure FrameArc(rect: Rect;  start, ending : integer);external c;
procedure PaintArc(rect: Rect;  start, ending : integer);external c;
procedure InvertArc(rect: Rect;  start, ending : integer);external c;
procedure EraseArc(rect: Rect;  start, ending : integer);external c;
procedure PaintCircle(x , y, radius : integer); external c;
procedure FrameCircle(x , y, radius : integer); external c;
procedure EraseCircle(x , y, radius : integer); external c;

procedure DrawString(s : string; x, y : integer);external c;
procedure QueryStringSize(s : string; var width, height : integer);external c;

procedure MoveTo(x, y : integer); external c;
procedure LineTo(x, y : integer); external c;

function  CreatePoly: Polygon; external c;
function  CopyPoly(poly: Polygon) : Polygon; external c;
procedure PolyAddPt(var poly : Polygon; x, y: integer); external c;
procedure PolyQueryPt(poly : Polygon; index : integer; var x, y : integer); external c;
procedure FramePoly(poly : Polygon); external c;
procedure ErasePoly(poly : Polygon); external c;
procedure PaintPoly(poly : Polygon); external c;
procedure InsetPoly(var poly: Polygon; dx, dy : integer); external c;
procedure OffsetPoly(var poly: Polygon; dx, dy: integer); external c;



